import React, { ChangeEvent, FC, memo, useState } from 'react';
import {
    Box,
    Table,
    Thead,
    Tbody,
    Tr,
    Th,
    Td,
    Select,
    Button,
    Flex,
    Spacer,
    Input,
    Center,
} from '@chakra-ui/react';
import { useTable, useSortBy, usePagination, useFilters, Column } from 'react-table';

interface ICustomTable {
    columns: Column[];
    data: any[];
}

const CustomTable: FC<ICustomTable> = ({ data, columns }) => {
    const {
        headerGroups,
        prepareRow,
        page,
        canPreviousPage,
        canNextPage,
        pageCount,
        gotoPage,
        nextPage,
        previousPage,
        setPageSize,
        state: { pageIndex, pageSize },
        setFilter,
    } = useTable(
        {
            columns,
            data,
            initialState: { pageSize: 10 },
        },
        useFilters,
        useSortBy,
        usePagination,
    );

    const [filterInput, setFilterInput] = useState('');

    const handleFilterChange = (event: ChangeEvent<HTMLInputElement>) => {
        const value = event.target.value || '';
        setFilter('name', value);
        setFilterInput(value);
    };

    return (
        <Flex direction="column" justifyContent="center" margin="10px">
            <Flex justifyContent="center">
                <Input
                    variant="filled"
                    textColor="darkgoldenrod"
                    focusBorderColor="blue.100"
                    backgroundColor="white"
                    value={filterInput}
                    onChange={handleFilterChange}
                    placeholder="Search name"
                />
            </Flex>
            <Table variant="unstyled" color="blue.100">
                <Thead color="darkgoldenrod" borderBottom="3px solid rgba(184, 134, 11, 1)">
                    {headerGroups.map((headerGroup, index) => {
                        return (
                            <Tr key={index}>
                                {headerGroup.headers.map((column) => (
                                    <Th
                                        textTransform="none"
                                        letterSpacing="normal"
                                        {...column.getSortByToggleProps()}
                                        key={column.id}
                                    >
                                        {column.render('Header')}
                                    </Th>
                                ))}
                            </Tr>
                        );
                    })}
                </Thead>
                <Tbody>
                    {page.map((row, index) => {
                        prepareRow(row);
                        return (
                            <Tr key={index}>
                                {row.cells.map((cell, index) => (
                                    <Td key={index}>{cell.render('Cell')}</Td>
                                ))}
                            </Tr>
                        );
                    })}
                </Tbody>
            </Table>
            <Flex className="pagination">
                <Flex>
                    <Button
                        color="blue.10"
                        backgroundColor="darkgoldenrod"
                        onClick={() => gotoPage(0)}
                        disabled={!canPreviousPage}
                        _hover={{ backgroundColor: 'goldenrod' }}
                    >
                        {'<<'}
                    </Button>
                    &nbsp;
                    <Button
                        color="blue.10"
                        backgroundColor="darkgoldenrod"
                        onClick={() => previousPage()}
                        disabled={!canPreviousPage}
                        _hover={{ backgroundColor: 'goldenrod' }}
                    >
                        {'<'}
                    </Button>
                    &nbsp;
                    <Button
                        color="blue.10"
                        backgroundColor="darkgoldenrod"
                        onClick={() => nextPage()}
                        disabled={!canNextPage}
                        _hover={{ backgroundColor: 'goldenrod' }}
                    >
                        {'>'}
                    </Button>
                    &nbsp;
                    <Button
                        color="blue.10"
                        backgroundColor="darkgoldenrod"
                        onClick={() => gotoPage(pageCount - 1)}
                        disabled={!canNextPage}
                        _hover={{ backgroundColor: 'goldenrod' }}
                    >
                        {'>>'}
                    </Button>
                </Flex>
                <Spacer />
                <Center color="darkgoldenrod">
                    Page {pageIndex + 1} of {pageCount || 1}
                </Center>
                <Spacer />
                <Box>
                    <Select
                        variant="filled"
                        backgroundColor="darkgoldenrod"
                        color="blue.10"
                        value={pageSize}
                        onChange={(e) => {
                            setPageSize(Number(e.target.value));
                        }}
                        _hover={{ backgroundColor: 'goldenrod' }}
                        _focus={{ backgroundColor: 'goldenrod' }}
                    >
                        {[5, 10, 20, 30, 40, 50].map((pageSize) => (
                            <option
                                key={pageSize}
                                value={pageSize}
                                style={{
                                    color: 'darkgoldenrod',
                                    backgroundColor: '#EEF8FC',
                                }}
                            >
                                Show {pageSize}
                            </option>
                        ))}
                    </Select>
                </Box>
            </Flex>
        </Flex>
    );
};

export default memo(CustomTable);
