import React, { FC, memo, useState } from 'react';
import {
    Box,
    Flex,
    FormControl,
    FormErrorMessage,
    FormLabel,
    Input,
    InputGroup,
    InputRightAddon,
    Select,
} from '@chakra-ui/react';
import DatePicker from 'react-datepicker';

import 'react-datepicker/dist/react-datepicker.css';
import { CalendarIcon } from '@chakra-ui/icons';
import moment from 'moment';

interface IFormDate {
    errorMessage: string;
    isInvalid: boolean;
    label: string;
    name: string;
    initialValue?: Date | string;
    placeholder: string;
    setFieldValue: Function;
    type?: string;
}

const FormDate: FC<IFormDate> = ({
    errorMessage,
    isInvalid,
    label,
    name,
    initialValue,
    placeholder,
    setFieldValue,
}) => {
    const today = moment().format('YYYY-MM-DD');
    const [newDate, setNewDate] = useState(new Date(initialValue || today));
    const range = (end: number, start: number) =>
        Array.from({ length: end - start }, (v, k) => end - k);
    const years = range(new Date().getFullYear(), 1969);
    const months = [
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December',
    ];

    const formStyling =
        name === 'date_of_birth'
            ? {
                  display: 'flex',
                  flexDirection: 'column' as 'column',
                  alignItems: 'center',
              }
            : {};

    const inputWidth = name === 'date_of_birth' ? '200px' : 'auto';

    return (
        <Box>
            <FormLabel
                htmlFor={name}
                textAlign="center"
                color="blue.100"
                fontWeight="bold"
                cursor="pointer"
            >
                {label}
            </FormLabel>
            <FormControl isInvalid={isInvalid} style={formStyling}>
                <DatePicker
                    renderCustomHeader={({ date, changeYear, changeMonth }) => (
                        <Flex
                            style={{
                                margin: 10,
                                display: 'flex',
                                justifyContent: 'center',
                                textAlign: 'center',
                            }}
                        >
                            <Select
                                value={date.getFullYear()}
                                onChange={(event) => changeYear(parseInt(event.target.value, 10))}
                            >
                                {years.map((option) => (
                                    <option key={option} value={option}>
                                        {option}
                                    </option>
                                ))}
                            </Select>

                            <Select
                                value={months[date.getMonth()]}
                                onChange={(e) => changeMonth(months.indexOf(e.target.value))}
                            >
                                {months.map((option) => (
                                    <option key={option} value={option}>
                                        {option}
                                    </option>
                                ))}
                            </Select>
                        </Flex>
                    )}
                    dateFormat="yyyy-MM-dd"
                    maxDate={new Date()}
                    withPortal
                    calendarStartDay={1}
                    selected={newDate}
                    placeholderText={placeholder}
                    onChange={(date: Date) => {
                        setNewDate(date);
                        return setFieldValue(name, date.toJSON().slice(0, 10));
                    }}
                    customInput={
                        <InputGroup style={{ textAlign: 'center', width: inputWidth }}>
                            <Input
                                error={isInvalid || 'false'}
                                value={newDate.toJSON().slice(0, 10)}
                                textAlign="center"
                                onChange={() => {}}
                            />
                            <InputRightAddon>
                                <CalendarIcon color="darkgoldenrod" />
                            </InputRightAddon>
                        </InputGroup>
                    }
                />
                <Box height="5">
                    <FormErrorMessage fontSize="xxs" marginTop="xxs" marginLeft="xs">
                        {errorMessage}
                    </FormErrorMessage>
                </Box>
            </FormControl>
        </Box>
    );
};

export default memo(FormDate);
