import React, { FC, memo, useCallback } from 'react';
import { useDisclosure } from '@chakra-ui/react';
import { Form, Formik } from 'formik';

import { actions, dispatch } from 'src/store';

import { Forms } from 'src/components';

import validationSchema from './validationSchema';
import ModalComponent from 'src/components/ModalComponent';

interface FormValues {
    player_id?: number;
    family_anamnesis: number[];
    player_anamnesis: number[];
    symptoms: number[];
}

interface IAnamnesisForm {
    buttonType: string;
    initialValues: FormValues;
    family_choices: string[];
    player_choices: string[];
    symptoms_choices: string[];
}

const AnamnesisForm: FC<IAnamnesisForm> = ({
    buttonType,
    initialValues,
    family_choices,
    player_choices,
    symptoms_choices,
}) => {
    const { isOpen, onOpen, onClose } = useDisclosure();
    const action = dispatch();
    const { addMedicalAnamnesis } = actions.medicalAnamnesis;

    const handleMedicalAnamnesisChange = useCallback(
        async (values) => {
            const response = await action(addMedicalAnamnesis(values));
            if (response.type === 'addMedicalAnamnesis/fulfilled') {
                onClose();
            }
        },
        [action, addMedicalAnamnesis, onClose],
    );

    return (
        <>
            <ModalComponent
                buttonType={buttonType}
                secondButton="submit"
                formID="modalAnamnesisForm"
                isOpen={isOpen}
                onOpen={onOpen}
                onClose={onClose}
            >
                <Formik
                    validationSchema={validationSchema}
                    initialValues={initialValues}
                    onSubmit={(values) => handleMedicalAnamnesisChange(values)}
                >
                    {({ errors, touched, handleChange, values }) => (
                        <Form id="modalAnamnesisForm" autoComplete="off">
                            <Forms.FormChoice
                                errorMessage={errors.family_anamnesis || ''}
                                isInvalid={
                                    (errors.family_anamnesis && touched.family_anamnesis) || false
                                }
                                choices={family_choices}
                                value={values.family_anamnesis}
                                label="family anamnesis"
                                name="family_anamnesis"
                                onChange={handleChange}
                            />
                            <Forms.FormChoice
                                errorMessage={errors.player_anamnesis || ''}
                                isInvalid={
                                    (errors.player_anamnesis && touched.player_anamnesis) || false
                                }
                                choices={player_choices}
                                value={values.player_anamnesis}
                                label="player anamnesis"
                                name="player_anamnesis"
                                onChange={handleChange}
                            />
                            <Forms.FormChoice
                                errorMessage={errors.symptoms || ''}
                                isInvalid={(errors.symptoms && touched.symptoms) || false}
                                choices={symptoms_choices}
                                value={values.symptoms}
                                label="symptoms"
                                name="symptoms"
                                onChange={handleChange}
                            />
                        </Form>
                    )}
                </Formik>
            </ModalComponent>
        </>
    );
};

export default memo(AnamnesisForm);
