import React, { FC, memo, useCallback, useEffect, useState } from 'react';
import { Box, Button, Flex, Link, Table, Tbody, Td, Tr } from '@chakra-ui/react';
import {
    AddIcon,
    ArrowForwardIcon,
    DeleteIcon,
    ExternalLinkIcon,
    MinusIcon,
} from '@chakra-ui/icons';
import { Form, Formik } from 'formik';

import { actions, dispatch } from 'src/store';

import { Forms } from 'src/components';

import validationSchema from './validationSchema';
import { MedicineData } from 'src/constants/interfaces/medicine';

interface FormValues {
    player_id?: number;
    name: string;
    TUE: string | Blob;
}

interface IMedicineForm {
    medicine?: Array<MedicineData>;
    initialValues: FormValues;
}

const MedicineForm: FC<IMedicineForm> = ({ medicine, initialValues }) => {
    const [open, setOpen] = useState(false);
    const [display, setDisplay] = useState('none');
    const action = dispatch();
    const { addMedicine, deleteMedicine } = actions.medicine;

    useEffect(() => {
        setDisplay(open ? 'visible' : 'none');
    }, [open]);

    const handleMedicineDelete = useCallback(
        (values) => {
            return action(deleteMedicine(values));
        },
        [action, deleteMedicine],
    );

    const handleMedicineChange = useCallback(
        async (values) => {
            return await action(addMedicine(values));
        },
        [action, addMedicine],
    );

    return (
        <>
            {medicine?.[0] && (
                <Table variant="unstyled" textColor="blue.100">
                    <Tbody>
                        {medicine.map((med) => {
                            return (
                                <Tr key={med.id}>
                                    <Td paddingY="2px" paddingRight="2px">
                                        {med.name}
                                        {med.TUE && (
                                            <>
                                                <ArrowForwardIcon marginInline="10px" />
                                                <Link href={`${med.TUE}`} isExternal>
                                                    TUE
                                                    <ExternalLinkIcon mx="2px" />
                                                </Link>
                                            </>
                                        )}
                                    </Td>
                                    <Td paddingY="5px" paddingLeft="0">
                                        <Button size="sm" onClick={() => handleMedicineDelete(med)}>
                                            <DeleteIcon
                                                textColor="darkgoldenrod"
                                                fontSize="medium"
                                            />
                                        </Button>
                                    </Td>
                                </Tr>
                            );
                        })}
                    </Tbody>
                </Table>
            )}
            <Box
                display={display}
                paddingY="10px"
                marginTop="10px"
                borderTop="2px"
                borderTopStyle="dotted"
                borderTopColor="blue.100"
            >
                <Formik
                    validationSchema={validationSchema}
                    initialValues={initialValues}
                    onSubmit={(values, { resetForm }) => {
                        const data = new FormData();
                        data.append('player_id', String(values.player_id));
                        data.append('name', values.name);
                        data.append('TUE', values.TUE);
                        handleMedicineChange(data).then((response) => {
                            if (response.type === 'addMedicine/fulfilled') {
                                resetForm();
                                setOpen(false);
                            }
                        });
                    }}
                >
                    {({ errors, touched, setFieldValue, handleSubmit, values }) => (
                        <Form id="modalMedicineForm" onSubmit={handleSubmit} autoComplete="off">
                            <Forms.FormField
                                errorMessage={errors.name || ''}
                                isInvalid={(errors.name && touched.name) || false}
                                value={values.name}
                                label="Name:"
                                name="name"
                                placeholder="name"
                                setFieldValue={setFieldValue}
                            />
                            <Flex position="relative" justifyContent="space-between">
                                <Forms.FormFile
                                    errorMessage={errors.TUE || ''}
                                    isInvalid={(errors.TUE && touched.TUE) || false}
                                    value={values.TUE}
                                    label="TUE:"
                                    name="TUE"
                                    type="file"
                                    placeholder="TUE"
                                    setFieldValue={setFieldValue}
                                />
                                <Button
                                    position="absolute"
                                    bottom="5"
                                    right="0"
                                    colorScheme="green"
                                    type="submit"
                                >
                                    Submit
                                </Button>
                            </Flex>
                        </Form>
                    )}
                </Formik>
            </Box>
            <Flex justifyContent="center">
                <Button color="blue.100" fontSize="large" onClick={() => setOpen(!open)}>
                    {open ? <MinusIcon /> : <AddIcon />}
                </Button>
            </Flex>
        </>
    );
};

export default memo(MedicineForm);
