import React, { FC, memo, useCallback } from 'react';
import { useDisclosure } from '@chakra-ui/react';
import { Form, Formik } from 'formik';

import { actions, dispatch } from 'src/store';

import { Forms } from 'src/components';

import validationSchema from './validationSchema';
import ModalComponent from 'src/components/ModalComponent';

interface FormValues {
    id?: number;
    name: string;
    position: string;
    leg: string;
    date_of_birth: Date;
    games: number;
    image: string | Blob;
}

interface IPlayerDetailsForm {
    buttonType: string;
    initialValues: FormValues;
}

const PlayerDetailsForm: FC<IPlayerDetailsForm> = ({ buttonType, initialValues }) => {
    const { isOpen, onOpen, onClose } = useDisclosure();
    const action = dispatch();
    const { editPlayer } = actions.player;

    const handlePlayerDetailsChange = useCallback(
        async (values, id) => {
            const response = await action(editPlayer({ formData: values, id }));
            if (response.type === 'editPlayer/fulfilled') {
                onClose();
            }
        },
        [action, editPlayer, onClose],
    );

    return (
        <>
            <ModalComponent
                buttonType={buttonType}
                secondButton="submit"
                formID="modalForm"
                isOpen={isOpen}
                onOpen={onOpen}
                onClose={onClose}
            >
                <Formik
                    validationSchema={validationSchema}
                    initialValues={initialValues}
                    onSubmit={(values) => {
                        const data = new FormData();

                        data.append('id', String(values.id));
                        data.append('name', values.name);
                        data.append('position', values.position);
                        data.append('leg', values.leg);
                        data.append('date_of_birth', values.date_of_birth.toString());
                        data.append('games', String(values.games));
                        data.append('image', values.image);

                        handlePlayerDetailsChange(data, values.id);
                    }}
                >
                    {({ errors, touched, initialValues, setFieldValue, handleSubmit }) => (
                        <Form id="modalForm" onSubmit={handleSubmit} autoComplete="off">
                            <Forms.FormField
                                errorMessage={errors.position || ''}
                                isInvalid={(errors.position && touched.position) || false}
                                label="Position:"
                                name="position"
                                placeholder="position"
                                initialValue={initialValues.position}
                                setFieldValue={setFieldValue}
                            />
                            <Forms.FormField
                                errorMessage={errors.leg || ''}
                                isInvalid={(errors.leg && touched.leg) || false}
                                label="Leg:"
                                name="leg"
                                placeholder="leg"
                                initialValue={initialValues.leg}
                                setFieldValue={setFieldValue}
                            />
                            <Forms.FormDate
                                errorMessage={String(errors.date_of_birth) || ''}
                                isInvalid={
                                    (errors.date_of_birth && !!touched.date_of_birth) || false
                                }
                                label="Date of birth:"
                                name="date_of_birth"
                                placeholder="date of birth"
                                initialValue={initialValues.date_of_birth}
                                setFieldValue={setFieldValue}
                            />
                            <Forms.FormField
                                errorMessage={errors.games || ''}
                                isInvalid={(errors.games && touched.games) || false}
                                label="Games:"
                                name="games"
                                placeholder="games"
                                initialValue={initialValues.games}
                                setFieldValue={setFieldValue}
                            />
                            <Forms.FormFile
                                errorMessage={String(errors.image) || ''}
                                isInvalid={(errors.image && !!touched.image) || false}
                                label="Image:"
                                name="image"
                                type="file"
                                placeholder="Image"
                                setFieldValue={setFieldValue}
                            />
                        </Form>
                    )}
                </Formik>
            </ModalComponent>
        </>
    );
};

export default memo(PlayerDetailsForm);
