import React, { FC, memo, useCallback, useEffect, useState } from 'react';
import { Box, Button, Flex } from '@chakra-ui/react';
import { Form, Formik } from 'formik';

import { Forms } from 'src/components';

import validationSchema from './validationSchema';
import { AddIcon, MinusIcon } from '@chakra-ui/icons';
import { actions, dispatch } from 'src/store';

interface FormValues {
    player_id?: number;
    disease: string;
    date: string;
}

interface IVaccinationForm {
    initialValues: FormValues;
}

const VaccinationForm: FC<IVaccinationForm> = ({ initialValues }) => {
    const [open, setOpen] = useState(false);
    const [display, setDisplay] = useState('none');
    const action = dispatch();
    const { addVaccine } = actions.vaccination;

    useEffect(() => {
        setDisplay(!open ? 'none' : 'visible');
    }, [open]);

    const handleVaccineChange = useCallback(
        async (values) => {
            return await action(addVaccine(values));
        },
        [action, addVaccine],
    );

    return (
        <>
            <Box
                display={display}
                paddingY="10px"
                marginTop="10px"
                borderTop="2px"
                borderTopStyle="dotted"
                borderTopColor="blue.100"
            >
                <Formik
                    validationSchema={validationSchema}
                    initialValues={initialValues}
                    onSubmit={(values, { resetForm }) => {
                        handleVaccineChange(values).then((response) => {
                            if (response.type === 'addVaccine/fulfilled') {
                                resetForm();
                                setOpen(false);
                            }
                        });
                    }}
                >
                    {({ errors, touched, setFieldValue, handleSubmit, values }) => (
                        <Form id="modalVaccinationForm" onSubmit={handleSubmit} autoComplete="off">
                            <Forms.FormField
                                errorMessage={errors.disease || ''}
                                isInvalid={(errors.disease && touched.disease) || false}
                                value={values.disease}
                                label="Disease:"
                                name="disease"
                                placeholder="disease"
                                setFieldValue={setFieldValue}
                            />
                            <Flex position="relative" justifyContent="space-between">
                                <Forms.FormDate
                                    errorMessage={errors.date || ''}
                                    isInvalid={(errors.date && touched.date) || false}
                                    label="Date:"
                                    name="date"
                                    placeholder="date"
                                    initialValue={initialValues.date}
                                    setFieldValue={setFieldValue}
                                />
                                <Button
                                    position="absolute"
                                    bottom="5"
                                    right="0"
                                    colorScheme="green"
                                    type="submit"
                                >
                                    Submit
                                </Button>
                            </Flex>
                        </Form>
                    )}
                </Formik>
            </Box>
            <Flex justifyContent="center">
                <Button color="blue.100" fontSize="large" onClick={() => setOpen(!open)}>
                    {!open && <AddIcon />}
                    {open && <MinusIcon />}
                </Button>
            </Flex>
        </>
    );
};

export default memo(VaccinationForm);
