import React, { FC, memo, useEffect, useState } from 'react';
import { useSelector } from 'react-redux';
import { useMatch } from 'react-router-dom';

import {
    Flex,
    Heading,
    ListItem,
    OrderedList,
    Table,
    Tbody,
    Td,
    Tr,
    useDisclosure,
    Link,
    Image,
} from '@chakra-ui/react';
import { ExternalLinkIcon } from '@chakra-ui/icons';

import { ListItems, Loader, ModalComponent, ModalDeleteForm } from 'src/components';
import { selectors } from 'src/store';

import { anamnesisChoices } from 'src/constants';
import { PlayerData } from 'src/constants/interfaces/player';

import PlayerDetailsForm from 'src/components/PlayerDetailsForm';
import GeneralOverviewForm from 'src/components/GeneralOverviewForm';
import MedicalAnamnesisForm from 'src/components/MedicalAnamnesisForm';
import MedicineForm from 'src/components/MedicineForm';
import moment from 'moment';

import blankProfileImage from 'src/images/blank-profile.png';

const PlayerDetails: FC = () => {
    const [player, setPlayer] = useState<PlayerData | undefined>();
    const match = useMatch('/medical/players/:id');
    const hasLoaded = useSelector(selectors.players.selectHasLoaded);
    const players = useSelector(selectors.players.selectPlayers);

    const today = moment().format('YYYY-MM-DD');
    const { isOpen, onOpen, onClose } = useDisclosure();
    const firstColumnWidth = '150px';
    const secondColumnWidth = '600px';

    useEffect(() => {
        if (match && players) {
            return setPlayer(players.find((player) => player?.id === Number(match.params.id)));
        }
    }, [hasLoaded, match, players]);

    if (!hasLoaded) {
        return <Loader />;
    }

    return (
        <Flex direction={{ sm: 'column', xl: 'row' }} justifyContent="space-between">
            <Flex
                height="fit-content"
                order={{ xl: 2 }}
                position={{ xl: 'sticky' }}
                top={{ xl: '50' }}
                justifyContent="center"
                marginBottom="20px"
            >
                <Image
                    src={player?.image || blankProfileImage}
                    width="300px"
                    height="max-content"
                />
            </Flex>
            <Flex direction="column" maxWidth="900px" order={{ xl: 1 }}>
                <Heading color="darkgoldenrod">{player?.name}</Heading>

                <Table
                    variant="unstyled"
                    color="blue.100"
                    marginTop="10px"
                    width="auto"
                    minWidth={{ xl: '640px', '2xl': '800px' }}
                >
                    <Tbody>
                        <Tr borderTop="solid" borderTopColor="blue.100">
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Position:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.position}
                            </Td>
                            <Td rowSpan={4}>
                                <Flex direction="column">
                                    <PlayerDetailsForm
                                        buttonType="pencil"
                                        initialValues={{
                                            id: player?.id,
                                            name: player?.name || '',
                                            position: player?.position || '',
                                            leg: player?.leg || '',
                                            date_of_birth: player?.date_of_birth || new Date(today),
                                            games: player?.games || 0,
                                            image: player?.image || '',
                                        }}
                                    />
                                </Flex>
                            </Td>
                        </Tr>
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Leg:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.leg}
                            </Td>
                        </Tr>
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Date of birth:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.date_of_birth}
                            </Td>
                        </Tr>
                        <Tr borderBottom="solid">
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Games:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.games}
                            </Td>
                        </Tr>
                        {player?.general_medical_overview[0] && (
                            <Tr
                                borderBottom="2px"
                                borderBottomStyle="dotted"
                                borderBottomColor="blue.100"
                            >
                                <Td colSpan={2} color="darkgoldenrod">
                                    {new Date(
                                        player?.general_medical_overview[0]?.time_made,
                                    ).toDateString()}
                                </Td>
                            </Tr>
                        )}
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Height:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {
                                    player?.general_medical_overview?.[
                                        player?.general_medical_overview.length - 1
                                    ]?.height
                                }
                                &nbsp;cm
                            </Td>
                            <Td rowSpan={3}>
                                <Flex direction="column">
                                    <GeneralOverviewForm
                                        buttonType="plus"
                                        initialValues={{
                                            player_id: player?.id,
                                            height:
                                                player?.general_medical_overview?.[
                                                    player?.general_medical_overview.length - 1
                                                ]?.height,
                                            weight:
                                                player?.general_medical_overview?.[
                                                    player?.general_medical_overview.length - 1
                                                ]?.weight,
                                            BP: '',
                                        }}
                                    />
                                    <Flex direction="column" marginTop="10px">
                                        <ListItems data={player?.general_medical_overview} />
                                    </Flex>
                                </Flex>
                            </Td>
                        </Tr>
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Weight:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {
                                    player?.general_medical_overview?.[
                                        player?.general_medical_overview.length - 1
                                    ]?.weight
                                }
                                &nbsp;kg
                            </Td>
                        </Tr>
                        <Tr borderBottom="solid">
                            <Td paddingRight="5px" maxWidth="100px">
                                BP:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {
                                    player?.general_medical_overview?.[
                                        player?.general_medical_overview.length - 1
                                    ]?.BP
                                }
                            </Td>
                        </Tr>
                        {player?.medical_anamnesis[0] && (
                            <Tr
                                borderBottom="2px"
                                borderBottomStyle="dotted"
                                borderBottomColor="blue.100"
                            >
                                <Td colSpan={2} color="darkgoldenrod">
                                    {new Date(
                                        player?.medical_anamnesis[0]?.time_made,
                                    ).toDateString()}
                                </Td>
                            </Tr>
                        )}
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Family anamnesis:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.medical_anamnesis?.[0] && (
                                    <OrderedList>
                                        {player?.medical_anamnesis[
                                            player?.medical_anamnesis.length - 1
                                        ].family_anamnesis.map((familyAnam) => {
                                            return (
                                                <ListItem key={familyAnam.value}>
                                                    {familyAnam.text}
                                                </ListItem>
                                            );
                                        })}
                                    </OrderedList>
                                )}
                            </Td>
                            <Td rowSpan={3}>
                                <Flex direction="column">
                                    <MedicalAnamnesisForm
                                        buttonType="plus"
                                        family_choices={anamnesisChoices.FAMILY_ANAMNESIS_DATA}
                                        player_choices={anamnesisChoices.PLAYER_ANAMNESIS_DATA}
                                        symptoms_choices={anamnesisChoices.SYMPTOMS_DATA}
                                        initialValues={{
                                            player_id: player?.id,
                                            family_anamnesis: [],
                                            player_anamnesis: [],
                                            symptoms: [],
                                        }}
                                    />
                                    <Flex direction="column" marginTop="10px">
                                        <ListItems data={player?.medical_anamnesis} />
                                    </Flex>
                                </Flex>
                            </Td>
                        </Tr>
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Player anamnesis:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.medical_anamnesis?.[0] && (
                                    <OrderedList>
                                        {player?.medical_anamnesis[
                                            player?.medical_anamnesis.length - 1
                                        ].player_anamnesis.map((playerAnam) => {
                                            return (
                                                <ListItem key={playerAnam.value}>
                                                    {playerAnam.text}
                                                </ListItem>
                                            );
                                        })}
                                    </OrderedList>
                                )}
                            </Td>
                        </Tr>
                        <Tr borderBottom="solid">
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Symptoms:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.medical_anamnesis?.[0] && (
                                    <OrderedList>
                                        {player?.medical_anamnesis[
                                            player?.medical_anamnesis.length - 1
                                        ].symptoms.map((symptom) => {
                                            return (
                                                <ListItem key={symptom.value}>
                                                    {symptom.text}
                                                </ListItem>
                                            );
                                        })}
                                    </OrderedList>
                                )}
                            </Td>
                        </Tr>
                        <Tr borderBottom="solid">
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Medicine:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.medicine?.[0] && (
                                    <OrderedList>
                                        {player?.medicine.map((med) => {
                                            return (
                                                <ListItem key={med.id}>
                                                    {med.name}
                                                    {med.TUE && (
                                                        <Link
                                                            href={`${med.TUE}`}
                                                            isExternal
                                                            marginLeft="20px"
                                                        >
                                                            TUE
                                                            <ExternalLinkIcon mx="2px" />
                                                        </Link>
                                                    )}
                                                </ListItem>
                                            );
                                        })}
                                    </OrderedList>
                                )}
                            </Td>
                            <Td>
                                <Flex direction="column">
                                    <ModalComponent
                                        buttonType="pencil"
                                        formID="modalMedicineForm"
                                        isOpen={isOpen}
                                        onOpen={onOpen}
                                        onClose={onClose}
                                    >
                                        <MedicineForm
                                            medicine={player?.medicine}
                                            initialValues={{
                                                player_id: player?.id,
                                                name: '',
                                                TUE: '',
                                            }}
                                        />
                                    </ModalComponent>
                                </Flex>
                            </Td>
                        </Tr>
                        <Tr>
                            <Td paddingRight="5px" maxWidth={firstColumnWidth}>
                                Vaccination:
                            </Td>
                            <Td paddingRight="5px" maxWidth={secondColumnWidth}>
                                {player?.vaccination?.[0] && (
                                    <OrderedList>
                                        {player?.vaccination.map((vaccine) => {
                                            return (
                                                <ListItem key={vaccine.id}>
                                                    {vaccine.date}
                                                    &emsp;
                                                    {vaccine.disease}
                                                </ListItem>
                                            );
                                        })}
                                    </OrderedList>
                                )}
                            </Td>
                            <Td>
                                <Flex direction="column">
                                    <ModalDeleteForm
                                        vaccination={player?.vaccination}
                                        initialValues={{
                                            player_id: player?.id,
                                            disease: '',
                                            date: today,
                                        }}
                                    />
                                </Flex>
                            </Td>
                        </Tr>
                    </Tbody>
                </Table>
            </Flex>
        </Flex>
    );
};

export default memo(PlayerDetails);
