import React, { FC, memo, useCallback } from 'react';
import { useSelector } from 'react-redux';
import { useNavigate } from 'react-router-dom';

import { Flex, Icon, IconButton, Menu, MenuButton, MenuItem, MenuList } from '@chakra-ui/react';

import { actions, dispatch, selectors } from 'src/store';
import { CustomTable, Loader } from 'src/components';

import { more } from 'src/components/icons';
import { paths } from 'src/constants';
import AddPlayerForm from 'src/components/AddPlayerForm';
import moment from 'moment';
import { TableInstance } from 'react-table';
import { PlayersData } from 'src/constants/interfaces/players';

const Players: FC = () => {
    const navigate = useNavigate();
    const hasLoaded = useSelector(selectors.players.selectHasLoaded);
    const players = useSelector(selectors.players.selectPlayers);
    const action = dispatch();
    const { addPlayer, deletePlayer } = actions.player;
    const today = moment().format('YYYY-MM-DD');

    const columns = [
        {
            Header: 'Name',
            accessor: 'name',
            sortType: 'basic',
        },
        {
            Header: 'Position',
            accessor: 'position',
            sortType: 'basic',
        },
        {
            Header: 'Leg',
            accessor: 'leg',
            sortType: 'basic',
        },
        {
            Header: 'Date of birth',
            accessor: 'date_of_birth',
            sortType: 'basic',
        },
        {
            Header: 'Games',
            accessor: 'games',
            sortType: 'basic',
        },
        {
            header: '',
            accessor: 'id',
            Cell: memo(function Cell(table: TableInstance) {
                return (
                    <Menu>
                        <MenuButton
                            as={IconButton}
                            icon={
                                <Icon
                                    as={more}
                                    style={{ transition: 'transform .2s ease-in-out' }}
                                />
                            }
                            variant="ghost"
                            _focus={{ outline: 'none' }}
                            _hover={{ '.chakra-icon': { transform: 'rotate(90deg) scale(1.5)' } }}
                        />
                        <MenuList minWidth="min-content">
                            <MenuItem
                                onClick={() =>
                                    navigate(
                                        paths.build(
                                            paths.PLAYER_DETAILS,
                                            table.cell.row.original.id,
                                        ),
                                    )
                                }
                            >
                                Details
                            </MenuItem>
                            <MenuItem
                                onClick={() => {
                                    const confirmed = confirm(
                                        `Are you sure you want to delete ${table.cell.row.original.name}?`,
                                    );
                                    return confirmed
                                        ? action(deletePlayer(table.cell.row.original.id))
                                        : null;
                                }}
                            >
                                Delete
                            </MenuItem>
                        </MenuList>
                    </Menu>
                );
            }),
        },
    ];

    const handleAddPlayer = useCallback((values) => action(addPlayer(values)), [action, addPlayer]);

    const sortById = (data: PlayersData) => {
        const sortedData = [...data];
        sortedData.sort((a, b) => (a.id || 0) - (b.id || 0));

        return sortedData;
    };

    if (!hasLoaded) {
        return <Loader />;
    }

    return (
        <Flex justifyContent="start" maxWidth="1000px" direction="column" margin="20px">
            <AddPlayerForm
                handleSubmit={handleAddPlayer}
                buttonType="plus"
                buttonText="&nbsp;Add Player"
                buttonColorScheme="green"
                initialValues={{
                    name: '',
                    position: '',
                    leg: '',
                    date_of_birth: today,
                    games: 0,
                }}
            />
            <CustomTable columns={columns} data={sortById(players)} />
        </Flex>
    );
};

export default memo(Players);
