from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.models import User

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.authtoken.models import Token

from rest_framework.mixins import (
    CreateModelMixin,
    RetrieveModelMixin,
    UpdateModelMixin,
    ListModelMixin,
)
from rest_framework.viewsets import GenericViewSet

from .models import (
    GeneralMedicalOverview,
    MedicalAnamnesis,
    Medicine,
    Player,
    Vaccination
)
from .serializers import (
    GeneralMedicalOverviewSerializer,
    MedicalAnamnesisSerializer,
    MedicineSerializer,
    PlayerSerializer,
    UserSerializer,
    VaccinationSerializer
)


class PlayerViewSet(CreateModelMixin, RetrieveModelMixin, UpdateModelMixin, ListModelMixin, GenericViewSet):
    serializer_class = PlayerSerializer
    queryset = Player.objects.all()

    def destroy(self, request, *args, **kwargs):
        id = kwargs.get('pk')
        player = Player.objects.get(id=id)
        responseData = PlayerSerializer(player).data
        player.delete()
        return Response(responseData)


class GeneralMedicalOverviewViewSet(RetrieveModelMixin, UpdateModelMixin, ListModelMixin, GenericViewSet):
    serializer_class = GeneralMedicalOverviewSerializer
    queryset = GeneralMedicalOverview.objects.all()

    def create(self, request, *args, **kwargs):
        player = Player.objects.get(id=request.data.pop('player_id'))
        general_medical_overview = GeneralMedicalOverview.objects.create(
            player=player,
            **request.data
        )
        return Response(GeneralMedicalOverviewSerializer(general_medical_overview).data)

    def destroy(self, request, *args, **kwargs):
        id = kwargs.get('pk')
        generalOverview = GeneralMedicalOverview.objects.get(id=id)
        responseData = GeneralMedicalOverviewSerializer(generalOverview).data
        generalOverview.delete()
        return Response(responseData)


class MedicalAnamnesisViewSet(RetrieveModelMixin, UpdateModelMixin, ListModelMixin, GenericViewSet):
    serializer_class = MedicalAnamnesisSerializer
    queryset = MedicalAnamnesis.objects.all()

    def create(self, request, *args, **kwargs):
        player = Player.objects.get(id=request.data.pop('player_id'))
        medical_anamnesis = MedicalAnamnesis.objects.create(
            player=player,
            **request.data
        )
        return Response(MedicalAnamnesisSerializer(medical_anamnesis).data)

    def destroy(self, request, *args, **kwargs):
        id = kwargs.get('pk')
        medicalAnamnesis = MedicalAnamnesis.objects.get(id=id)
        responseData = MedicalAnamnesisSerializer(medicalAnamnesis).data
        medicalAnamnesis.delete()
        return Response(responseData)


class MedicineViewSet(RetrieveModelMixin, UpdateModelMixin, ListModelMixin, GenericViewSet):
    serializer_class = MedicineSerializer
    queryset = Medicine.objects.all()

    def create(self, request, *args, **kwargs):
        player_id = int(request.data.get('player_id'))
        player = Player.objects.get(id=player_id)
        medicine = Medicine.objects.create(
            player=player,
            name=request.data.get('name'),
            TUE=request.data.get('TUE')
        )
        return Response(MedicineSerializer(medicine).data)

    def destroy(self, request, *args, **kwargs):
        id = kwargs.get('pk')
        medicine = Medicine.objects.get(id=id)
        responseData = MedicineSerializer(medicine).data
        medicine.delete()
        return Response(responseData)


class VaccinationViewSet(RetrieveModelMixin, UpdateModelMixin, ListModelMixin, GenericViewSet):
    serializer_class = VaccinationSerializer
    queryset = Vaccination.objects.all()

    def create(self, request, *args, **kwargs):
        player = Player.objects.get(id=request.data.pop('player_id'))
        vaccination = Vaccination.objects.create(
            player=player,
            **request.data
        )
        return Response(VaccinationSerializer(vaccination).data)

    def destroy(self, request, *args, **kwargs):
        id = kwargs.get('pk')
        vaccination = Vaccination.objects.get(id=id)
        responseData = VaccinationSerializer(vaccination).data
        vaccination.delete()
        return Response(responseData)


@api_view(['GET'])
def token_restricted_access(request, path):
    response = HttpResponse()
    # Content-type will be detected by nginx
    del response['Content-Type']
    response['X-Accel-Redirect'] = f'/restricted/{path}'
    print(f'Response {response}')
    return response


def restricted_access(request, path):
    access_granted = False
    user = request.user
    if user.is_authenticated:
        if user.is_staff or user.is_superuser:
            # If admin, everything is granted
            access_granted = True

    if access_granted:
        response = HttpResponse()
        # Content-type will be detected by nginx
        del response['Content-Type']
        response['X-Accel-Redirect'] = f'/restricted/{path}'
        print(f'Response {response}')
        return response
    else:
        return HttpResponseForbidden('Not authorized to access this media.')


@api_view(['POST'])
def health(request):
    token = request.headers['Authorization'].split('Token ')[1]
    try:
        user = Token.objects.get(key=token).user
        if user.is_authenticated:
            return Response(UserSerializer(user).data)
    except Token.DoesNotExist:
        return Response({'error': 'Invalid token'}, status=401)
